#!/usr/bin/env python3

import os
import subprocess
import sys


def run_command_in_folder(command, folder):
    """Run a bash command in a specific folder."""
    run_command = subprocess.Popen(command,
                                   shell=True,
                                   cwd=folder,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   universal_newlines=True)
    stdout, _ = run_command.communicate()
    command_output = stdout.rstrip()
    return command_output


def get_git_repo_root(some_folder_in_root_repo='./'):
    """Get the root folder of the current git repository."""
    return run_command_in_folder('git rev-parse --show-toplevel', some_folder_in_root_repo)


def get_linter_folder():
    """Find the folder where this linter is stored."""
    try:
        return os.environ['AMZ_LINT']
    except KeyError:
        print("Cannot find linter because the environment variable AMZ_LINT doesn't exist.")
        sys.exit(1)


def main():
    # Get git root folder.
    repo_root = get_git_repo_root()

    # Get linter subfolder
    linter_folder = get_linter_folder()

    # Append linter folder to the path so that we can import the linter module.
    linter_folder = os.path.join(repo_root, linter_folder)
    sys.path.append(linter_folder)

    import linter

    # Read linter config file.
    home_dir = os.environ["HOME"]
    linter_config_file = home_dir + '/.amz/data/linterconfig_pylint3.yaml'

    linter.linter_check(repo_root, linter_folder, linter_config_file)


if __name__ == "__main__":
    main()



# Handles install routine - `amz install`
import subprocess as sp
import os

from amz.loading import Loading


def apt_deps(filepath, verbose=False):

    filepath_display = filepath
    # Check file exists
    try:
        if os.path.exists(filepath):
            if len(filepath) > 30:
                filepath_display = f'...{filepath[-25:]}'
        else:
            print(f'{filepath} - file does not exist')
            print('Exiting...')
            return
    except:
        return

    l = Loading(f"Installing apt dependencies from - {filepath_display}")
    l.end(-2)

    p0 = sp.run('apt list --installed', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
    apt_exist_list = [x.split("/")[0].lower() for x in p0.stdout.decode().split('\n')]
    apt_dep_list = open(filepath, 'r').read().split('\n')

    # Refine Lists not to contain comments(#) or empty lines
    apt_dep_list = [pkg for pkg in apt_dep_list if pkg != '' and '#' not in pkg]

    for package in apt_dep_list:
        if package.lower() in apt_exist_list:
            if not verbose:
                l.chain(f'{package} already installed', '\t')
                l.end(0)
            elif verbose:
                print(f'{package} already installed')
        elif package.lower() not in apt_exist_list:
            if not verbose: l.chain(f'{package}', '\t')
            p = sp.run(f'sudo apt --yes install {package}',
                       shell=True,
                       stdout=(None if verbose else sp.PIPE),
                       stderr=(None if verbose else sp.PIPE))
            if not verbose: l.end(p.returncode)


def py2_deps(filepath, verbose=False):

    filepath_display = filepath
    # Check file exists
    try:
        if os.path.exists(filepath):
            if len(filepath) > 30:
                filepath_display = f'...{filepath[-25:]}'
        else:
            print(f'{filepath} - file does not exist')
            print('Exiting...')
            return
    except:
        return

    l = Loading(f"Installing Py2 dependencies from - {filepath_display}")
    l.end(-2)

    p0 = sp.run('pip list', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
    py2_exist_list = [x.split(" ")[0].lower() for x in p0.stdout.decode().split('\n')]
    py2_dep_list = open(filepath, 'r').read().split('\n')

    # Refine Lists not to contain comments(#) or empty lines
    py2_dep_list = [pkg for pkg in py2_dep_list if pkg != '' and '#' not in pkg]

    for package in py2_dep_list:
        if package.lower() in py2_exist_list:
            if not verbose:
                l.chain(f'{package} already installed', '\t')
                l.end(0)
            elif verbose:
                print(f'{package} already installed')
        elif package.lower() not in py2_exist_list:
            if not verbose: l.chain(f'{package}', '\t')
            p = sp.run(f'pip install {package}',
                       shell=True,
                       stdout=(None if verbose else sp.PIPE),
                       stderr=(None if verbose else sp.PIPE))
            if not verbose: l.end(p.returncode)


def py3_deps(filepath, verbose=False):

    filepath_display = filepath
    # Check file exists
    try:
        if os.path.exists(filepath):
            if len(filepath) > 30:
                filepath_display = f'...{filepath[-25:]}'
        else:
            print(f'{filepath} - file does not exist')
            print('Exiting...')
            return
    except:
        return

    l = Loading(f"Installing Py3 dependencies from - {filepath_display}")
    l.end(-2)

    p0 = sp.run('pip3 list', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
    py3_exist_list = [x.split(" ")[0].lower() for x in p0.stdout.decode().split('\n')]
    py3_dep_list = open(filepath, 'r').read().split('\n')

    # Refine Lists not to contain comments(#) or empty lines
    py3_dep_list = [pkg for pkg in py3_dep_list if pkg != '' and '#' not in pkg]

    for package in py3_dep_list:
        if package.lower() in py3_exist_list:
            if not verbose:
                l.chain(f'{package} already installed', '\t')
                l.end(0)
            elif verbose:
                print(f'{package} already installed')
        elif package.lower() not in py3_exist_list:
            if not verbose: l.chain(f'{package}', '\t')
            p = sp.run(f'pip3 install {package}',
                       shell=True,
                       stdout=(None if verbose else sp.PIPE),
                       stderr=(None if verbose else sp.PIPE))
            if not verbose: l.end(p.returncode)


def script(filepath, verbose=False):
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Disable pylint invalid module name complaint.
# pylint : disable = C0103
""" Pre-commit script for invoking linters and auto-formatters."""

# Make sure we use the new print function
from __future__ import print_function

import datetime
import imp
import os
import re
import subprocess
import distutils
import sys

import pylint.lint
import yaml

CLANG_FORMAT_DIFF_EXECUTABLE_VERSIONS = [
    "clang-format-diff", "clang-format-diff-6.0", "clang-format-diff-5.0", "clang-format-diff-4.0",
    "clang-format-diff-3.9", "clang-format-diff-3.8"
]

YAPF_FORMAT_EXECUTABLE = "yapf"

DEFAULT_CONFIG = {
    # Enable code checks and formatting by default.
    'use_clangformat': True,
    'use_cpplint': True,
    'use_yapf': True,
    'use_pylint': True,
    # Block commits that don't pass by default
    'block_commits': True,
    # Check all staged files by default.
    'whitelist': []
}


def get_user_confirmation(default=False):
    """Requests confirmation string from user"""
    sys.stdin = open('/dev/tty')
    while (True):
        resp = raw_input()
        try:
            if resp == '':
                return default
            elif distutils.util.strtobool(resp):
                return True
            else:
                return False
        except ValueError:
            print('{} is not a valid response.'.format(resp))
            print('Please answer with y(es) or n(o)')


def read_linter_config(filename):
    """Parses yaml config file."""

    config = DEFAULT_CONFIG
    with open(filename, 'r') as ymlfile:
        parsed_config = yaml.load(ymlfile, Loader=yaml.FullLoader)

    if 'clangformat' in parsed_config.keys():
        config['use_clangformat'] = parsed_config['clangformat']
    if 'cpplint' in parsed_config.keys():
        config['use_cpplint'] = parsed_config['cpplint']
    if 'yapf' in parsed_config.keys():
        config['use_yapf'] = parsed_config['yapf']
    if 'pylint' in parsed_config.keys():
        config['use_pylint'] = parsed_config['pylint']
    if 'block_commits' in parsed_config.keys():
        config['block_commits'] = parsed_config['block_commits']
    if 'whitelist' in parsed_config.keys():
        config['whitelist'] = parsed_config['whitelist']

    return config


def run_command_in_folder(command, folder):
    """Run a bash command in a specific folder."""
    run_command = subprocess.Popen(command,
                                   shell=True,
                                   cwd=folder,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   universal_newlines=True)
    stdout, _ = run_command.communicate()
    command_output = stdout.rstrip()
    return command_output


def get_number_of_commits(some_folder_in_root_repo='./'):
    command = (
        "git shortlog -s -n --all --author=\"$(git config " +
        "--get user.email)\" | grep -o '[0-9]\+' |  " +  # pylint: disable=W1401
        "awk '{s+=$1} END {print s}'")
    num_commits = run_command_in_folder(command, some_folder_in_root_repo)
    if not num_commits:
        num_commits = "0"
    return num_commits


def get_staged_files(some_folder_in_root_repo='./'):
    """Get all staged files from git."""
    output = run_command_in_folder("git diff --staged --name-only", some_folder_in_root_repo)
    if not output:
        return []
    else:
        return output.split("\n")


def get_unstaged_files(some_folder_in_root_repo='./'):
    """Get all unstaged files from git."""
    output = run_command_in_folder("git diff --name-only", some_folder_in_root_repo)
    return output.split("\n")


def check_cpp_lint(staged_files, cpplint_file, ascii_art, repo_root):
    """Runs Google's cpplint on all C++ files staged for commit,"""
    cpplint = imp.load_source('cpplint', cpplint_file)
    cpplint._cpplint_state.SetFilters('-legal/copyright,-build/c++11')  # pylint: disable=W0212

    # Prevent cpplint from writing to stdout directly, instead
    # the errors will be stored in pplint.output as (line, message) tuples.
    cpplint.print_stdout = False

    total_error_count = 0
    for changed_file in staged_files:
        if not os.path.isfile(changed_file):
            continue
        if changed_file.lower().endswith(('.cc', '.h', '.cpp', '.cu', '.cuh')):
            # Search iteratively for the root of the catkin package.
            package_root = ''
            search_dir = os.path.dirname(os.path.abspath(changed_file))
            found_package_root = False
            MAX_DEPTH_OF_FILES = 100
            for _ in range(1, MAX_DEPTH_OF_FILES):
                if os.path.isfile(search_dir + '/package.xml'):
                    package_root = search_dir
                    found_package_root = True
                    break
                # Stop if the root of the git repo is reached.
                if os.path.isdir(search_dir + '/.git'):
                    break
                search_dir = os.path.dirname(search_dir)
            assert found_package_root, ("Could not find the root of the "
                                        "catkin package that contains: "
                                        "{}".format(changed_file))

            # Get relative path to repository root.
            if os.path.isfile(os.path.join(repo_root, '.git')):
                # Repo is a submodule, look for parent git repository containing
                # this submodule.
                search_path = repo_root
                found_to_level_git_repo = False
                for _ in range(10):
                    search_path = os.path.dirname(repo_root)
                    if os.path.isdir(os.path.join(search_path, '.git')):
                        found_to_level_git_repo = True
                        break

                assert found_to_level_git_repo
                repo_root = search_path

            common_prefix = os.path.commonprefix(
                [os.path.abspath(repo_root),
                 os.path.abspath(package_root)])
            package_root = os.path.relpath(package_root, common_prefix)

            # The package root needs to be relative to the (top-level) repo
            # root. Otherwise the header guard logic will fail!
            cpplint._root = package_root + '/include'  # pylint: disable=W0212

            # Reset error count and messages:
            cpplint.output = []
            cpplint._cpplint_state.ResetErrorCounts()  # pylint: disable=W0212
            v_level = cpplint._cpplint_state.verbose_level  # pylint: disable=W0212

            cpplint.ProcessFile(changed_file, v_level)

            error_count = cpplint._cpplint_state.error_count  # pylint: disable=W0212
            if error_count > 0:
                total_error_count += error_count

                print("-" * 80)
                print("Found {} errors in : {}".format(error_count, changed_file))
                print("-" * 80)
                for line in cpplint.output:
                    assert len(line) == 2
                    print("line {: >4}:\t {}".format(line[0], line[1]))

    if total_error_count > 0:
        print("=" * 80)
        print("Found {} cpplint errors".format(total_error_count))
        print("=" * 80)

        if total_error_count > 50:
            print(ascii_art.AsciiArt.cthulhu)
        elif total_error_count > 20:
            print(ascii_art.AsciiArt.tiger)
        return False
    else:
        return True


def check_modified_after_staging(staged_files):
    """Checks if one of the staged files was modified after staging."""
    files_changed = get_unstaged_files()
    files_changed = filter(None, files_changed)

    staged_files_changed = 0

    staged_files_changed_list = []

    is_first = True
    for changed_file in files_changed:
        if changed_file in staged_files:

            if is_first:
                print("=" * 80)
                is_first = False

            print("\'{}\' was modified after staging".format(changed_file))
            staged_files_changed_list.append(changed_file)
            staged_files_changed += 1

    if staged_files_changed > 0:
        print("-" * 80)
        print("Found {} files that have been changed after staging".format(staged_files_changed))
        print("=" * 80)

    return staged_files_changed_list


def check_commit_against_master(repo_root):
    """Check if the current commit is intended to for the master branch."""
    current_branch = run_command_in_folder("git branch | grep \"*\" | sed \"s/* //\"", repo_root)
    # print("\nCurrent_branch: {}\n".format(current_branch))
    return current_branch == "master"


def check_if_merge_commit(repo_root):
    """Check if the current commit is a merge commit."""
    merge_msg_file_path = repo_root + "/.git/MERGE_MSG"
    return os.path.isfile(merge_msg_file_path)


def find_clang_format_executable():
    for executable in CLANG_FORMAT_DIFF_EXECUTABLE_VERSIONS:
        if subprocess.call("type " + executable,
                           shell=True,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE,
                           universal_newlines=True) == 0:
            return executable

    print("ERROR: clang-format-diff is not installed!")
    exit(1)


def run_clang_format(repo_root, staged_files, list_of_changed_staged_files):
    """Runs clang format on all cpp files staged for commit."""

    clang_format_path = ("/tmp/" + os.path.basename(os.path.normpath(repo_root)) + "_" +
                         datetime.datetime.now().isoformat() + ".clang.patch")

    clang_format_diff_executable = find_clang_format_executable()

    run_command_in_folder(
        "git diff -U0 --cached | " + clang_format_diff_executable + " -style=file -p1 > " +
        clang_format_path, repo_root)

    if not os.stat(clang_format_path).st_size == 0:
        if list_of_changed_staged_files:
            print("=" * 80)
            print("Cannot format your code, because some files are \n"
                  "only partially staged! Format your code or try \n"
                  "stashing your unstaged changes...")
            print("=" * 80)
            exit(1)

        run_command_in_folder("git apply -p0 " + clang_format_path, repo_root)
        run_command_in_folder("git add " + ' '.join(staged_files), repo_root)

        print("=" * 80)
        print("Formatted staged C++ files with clang-format.\n" +
              "Patch: {}".format(clang_format_path))
        print("=" * 80)
    return True


def run_yapf_format(repo_root, staged_files, list_of_changed_staged_files):
    """Runs yapf format on all python files staged for commit."""

    first_file_formatted = True
    for staged_file in staged_files:
        if not os.path.isfile(staged_file):
            continue
        if staged_file.endswith((".py")):

            # Check if the file needs formatting by applying the formatting and
            # store the results into a patch file.
            yapf_format_path = ("/tmp/" + os.path.basename(os.path.normpath(repo_root)) + "_" +
                                datetime.datetime.now().isoformat() + ".yapf.patch")
            task = (YAPF_FORMAT_EXECUTABLE +
                    " --style='{based_on_style: pep8, column_limit: 100}' -d " + staged_file +
                    " > " + yapf_format_path)
            run_command_in_folder(task, repo_root)

            if not os.stat(yapf_format_path).st_size == 0:
                if first_file_formatted:
                    print("=" * 80)
                    print("Formatted staged python files with pep8 style:")
                    first_file_formatted = False

                print("-> " + staged_file)

                if staged_file in list_of_changed_staged_files:
                    print("Cannot format your code, because this file is \n"
                          "only partially staged! Format your code or try \n"
                          "stashing your unstaged changes...")
                    print("=" * 80)
                    exit(1)
                else:
                    run_command_in_folder("git apply -p0 " + yapf_format_path, repo_root)
                    run_command_in_folder("git add " + staged_file, repo_root)

    if not first_file_formatted:
        print("=" * 80)
    return True


def check_python_lint(repo_root, staged_files, pylint_file):
    """Runs pylint on all python scripts staged for commit."""
    class TextReporterBuffer(object):
        """Stores the output produced by the pylint TextReporter."""
        def __init__(self):
            """init"""
            self.content = []

        def write(self, input_str):
            """write"""
            self.content.append(input_str)

        def read(self):
            """read"""
            return self.content

    is_python2 = True if sys.version[:4] == '2.7.' else False

    # Parse each pylint output line individualy and searches
    # for errors in the code.
    pylint_errors = []
    for changed_file in staged_files:
        if not os.path.isfile(changed_file):
            continue
        if re.search(r'\.py$', changed_file):
            # Skip empty files, like __init__.py
            if os.stat(changed_file).st_size == 0:
                continue

            # Apply the linter only to files using the same python version as of pylint
            with open(changed_file, 'r') as f:
                shebang_line = f.readline()
            if '#!/usr/bin/env' in shebang_line:
                if 'python3' in shebang_line and is_python2:
                    continue
                elif 'python2' in shebang_line and not is_python2:
                    continue
            else:
                raise RuntimeError('Missing shebang line in file: ' + changed_file)

            print("Running pylint on \'{}\'".format(repo_root + "/" + changed_file))
            pylint_output = TextReporterBuffer()
            pylint_args = ["--rcfile=" + pylint_file, "-rn", repo_root + "/" + changed_file]
            from pylint.reporters.text import TextReporter
            pylint.lint.Run(pylint_args, reporter=TextReporter(pylint_output), exit=False)

            for output_line in pylint_output.read():
                if re.search(r'^(E|C|W):', output_line):
                    print(changed_file + ": " + output_line)
                    pylint_errors.append(output_line)

    if pylint_errors:
        print("=" * 80)
        print("Found {} pylint errors".format(len(pylint_errors)))
        print("=" * 80)
        return False
    else:
        return True


def get_whitelisted_files(repo_root, files, whitelist):
    whitelisted = []

    for file_name in files:
        for entry in whitelist:
            # Add trailing slash if its a directory and no slash is already
            # there.
            if os.path.isdir(repo_root + '/' + entry):
                entry = os.path.join(os.path.normpath(entry), '')

            # Check if the file itself or its parent directory is in the
            # whitelist.
            if (file_name == entry or os.path.commonprefix([file_name, entry]) == entry):
                whitelisted.append(file_name)
                break

    return whitelisted


def linter_check(repo_root, linter_subfolder, linter_config_file=None):
    """ Main pre-commit function for calling code checking script. """

    if linter_config_file is not None and os.path.isfile(linter_config_file):
        print("Found repo linter config: {}".format(linter_config_file))
        linter_config = read_linter_config(linter_config_file)
    else:
        linter_config = DEFAULT_CONFIG

    cpplint_file = os.path.join(linter_subfolder, "cpplint.py")
    ascii_art_file = os.path.join(linter_subfolder, "ascii_art.py")

    if os.path.isfile(os.path.join(repo_root, ".pylintrc")):
        pylintrc_file = os.path.join(repo_root, ".pylintrc")
    else:
        pylintrc_file = os.path.join(linter_subfolder, "pylint.rc")
    print("Using pylint config file: {}".format(pylintrc_file))

    # print("Found linter subfolder: {}".format(linter_subfolder))
    # print("Found ascii art file at: {}".format(ascii_art_file))
    # print("Found cpplint file at: {}".format(cpplint_file))
    # print("Found pylint config file at: {}".format(pylintrc_file))

    # Run checks
    staged_files = get_staged_files()

    if not staged_files:
        print("\n")
        print("=" * 80)
        print("No files staged...")
        print("=" * 80)
        exit(1)

    if linter_config['whitelist']:
        whitelisted_files = get_whitelisted_files(repo_root, staged_files,
                                                  linter_config['whitelist'])
    else:
        whitelisted_files = staged_files

    # Load ascii art.
    ascii_art = imp.load_source('ascii_art', ascii_art_file)

    if check_commit_against_master(repo_root):
        print(ascii_art.AsciiArt.grumpy_cat)
        exit(1)

    if not check_if_merge_commit(repo_root):
        # Do not allow commiting files that were modified after staging. This
        # avoids problems such as forgetting to stage fixes of cpplint
        # complaints.
        list_of_changed_staged_files = check_modified_after_staging(whitelisted_files)

        if linter_config['use_clangformat']:
            run_clang_format(repo_root, whitelisted_files, list_of_changed_staged_files)

        if linter_config['use_yapf']:
            run_yapf_format(repo_root, whitelisted_files, list_of_changed_staged_files)

        # Use Google's C++ linter to check for compliance with Google style
        # guide.
        if linter_config['use_cpplint']:
            cpp_lint_success = check_cpp_lint(whitelisted_files, cpplint_file, ascii_art, repo_root)
        else:
            cpp_lint_success = True

        # Use pylint to check for comimpliance with Tensofrflow python
        # style guide.
        if linter_config['use_pylint']:
            pylint_success = check_python_lint(repo_root, whitelisted_files, pylintrc_file)
        else:
            pylint_success = True

        if not (cpp_lint_success and pylint_success):
            print("=" * 80)
            print("Commit not up to standards!")
            print("Please address the linter errors above.")
            print("=" * 80)
            if linter_config['block_commits']:
                exit(1)

            else:
                print("However, you may commit anyway, if you must.")
                print("All of these linter errors must be resolved before merge.")
                print("Would you like to commit anyway? yN")

                if get_user_confirmation():
                    print(ascii_art.AsciiArt.yoda)

                else:
                    exit(1)

        else:

            commit_number = get_number_of_commits(repo_root)
            lucky_commit = ((int(commit_number) + 1) % 42 == 0)

            if lucky_commit:
                print(ascii_art.AsciiArt.story)

                # print("=" * 80)
                print("Commit accepted, well done! This is your {}th commit!".format(commit_number))
                print("This is a lucky commit! " + "Please enjoy this free sheep story.")
                # print("=" * 80)
            else:
                print(ascii_art.AsciiArt.commit_success)

                # print("=" * 80)
                print("Commit accepted, well done! This is the {}th commit!".format(commit_number))
                # print("=" * 80)
    else:
        print(ascii_art.AsciiArt.homer_woohoo)



# Defines routine for `amz init`
import os, getpass
from distutils.dir_util import copy_tree
from distutils.file_util import copy_file
import yaml
import subprocess as sp

from amz.loading import Loading
import amz.meta as meta
import amz.install as install

HOME_DIR = os.environ["HOME"]
PKG_DIR = os.path.dirname(__file__)
AMZ_DATA_DIR = HOME_DIR + '/.amz/data'


# General Initialization
def amz_gen_init(verbose=False):

    l = Loading("AMZ tool initialization routine")
    l.end(-2)

    # Check for `~/.amz` directory and make one if not there
    l.chain("Checking for `~/.amz` directory", "\t")
    AMZ_DIR_PRESENT = os.path.exists(f"{HOME_DIR}/.amz")
    if not AMZ_DIR_PRESENT:
        os.mkdir(f"{HOME_DIR}/.amz")

    # Copy data folder if contents do not match
    l.chain("Copying data", "\t", 0)
    try:
        copy_tree(f'{PKG_DIR}/data', f'{HOME_DIR}/.amz/data')
    except FileExistsError:
        l.end(1)

    # Update version info in amz-config yaml
    try:
        amz_user_config = yaml.load(open(f'{AMZ_DATA_DIR}/amz-config.yaml'), Loader=yaml.FullLoader)
        amz_user_config['amz_tool_meta']['version'] = open(f'{AMZ_DATA_DIR}/VERSION').readlines()[0]
        amz_user_config['amz_tool_meta']['user'] = getpass.getuser()
        with open(f'{AMZ_DATA_DIR}/amz-config.yaml', 'w') as f:
            yaml.dump(amz_user_config, f)
            f.close()
    except:
        l.end(1)

    # Source `data/amz_source.sh` at the end of bashrc if it's not present
    l.chain("Appending `source ~/.amz/data/amz_source.sh` to bashrc", "\t", 0)

    BASHRC_LINE_EXISTS = "source ~/.amz/data/amz_source.sh\n" in open(f"{HOME_DIR}/.bashrc", 'r').readlines()

    if not BASHRC_LINE_EXISTS:
        with open(f"{HOME_DIR}/.bashrc", 'a') as f:
            f.write("\n# AMZ Tool Helper \nsource ~/.amz/data/amz_source.sh\n")
            f.close()

    l.end(0)


# Repo Initialization
def amz_repo_init(verbose=False):

    l = Loading("Initializing AMZ Repo")
    l.end(-2)

    l.chain("Preliminary Checks - are you inside the repo root? ", "\t")
    l.end(-3)

    # Check if `amz init gen` has been executed
    AMZ_INIT_EXECUTED = os.path.exists(f"{HOME_DIR}/.amz/data/amz_source.sh")
    if not AMZ_INIT_EXECUTED:
        print("Run `amz init gen` first!")
        l.end(1)
        return

    # Ask if user is in AMZ_ROOT
    while True:
        IN_AMZ_ROOT = input("\t\t (y/n): ").lower()
        if IN_AMZ_ROOT == 'n':
            print("Please run this command from the repo root")
            l.end(1)
            return
        elif IN_AMZ_ROOT not in ['y', 'n']:
            print("Didn't quite catch you there!")
        else:
            break

    # Check if actually inside AMZ Repo
    if not os.path.exists('amz.yaml'):
        print("You are lying! Not in amz repo root")
        l.end(1)
        return

    # Fetch some data and write into amz-config YAML file
    l.chain("Configuring the repo", "\t", 0)

    # Set cur dir as AMZ Root and set git repo root
    amz_root_dir = os.getcwd()
    auto_repo = False
    try:
        amz_git_remote = open(f'{amz_root_dir}/.git/config', 'r').readlines()[6][7:-1]
        if 'autonomous' in amz_git_remote:
            auto_repo = True
    except:
        print("Is it a git repo??? and has a valid remote?")
        l.end(1)
        return

    if auto_repo:

        try:
            amz_user_config = yaml.load(open(f'{HOME_DIR}/.amz/data/amz-config.yaml'), Loader=yaml.FullLoader)
            amz_user_config['amz_repo_meta']['amz_root'] = amz_root_dir
            amz_user_config['amz_repo_meta']['amz_repo_git_src'] = amz_git_remote
            amz_user_config['amz_repo_meta']['amz_repo_config'] = f'{amz_root_dir}/amz.yaml'
            with open(f'{AMZ_DATA_DIR}/amz-config.yaml', 'w') as f:
                yaml.dump(amz_user_config, f)
                f.close()
        except:
            print("Some issue with data files... Run `amz init gen` again. ")
            l.end(1)
            return

        # Update `amz_source.sh` with AMZ_ROOT
        meta.update_line_in_file('export AMZ_ROOT=', f"export AMZ_ROOT={amz_root_dir}", f"{AMZ_DATA_DIR}/amz_source.sh")
        meta.update_line_in_file('export AMZ_DATA=', f"export AMZ_DATA={AMZ_DATA_DIR}", f"{AMZ_DATA_DIR}/amz_source.sh")

    # Setup githooks and Linter
    l.chain("Installing Githooks", "\t", 0)
    try:
        # Ensure to be in the amz root
        os.chdir(amz_root_dir)
        copy_file(f'{AMZ_DATA_DIR}/githooks/pre-commit', f'{amz_root_dir}/.git/hooks/pre-commit', link='hard')
    except:
        l.end(1)

    l.chain("Installing Linter", "\t", 0)
    try:
        # Ensure to be in the amz root
        os.chdir(amz_root_dir)
        meta.update_line_in_file('export AMZ_LINT=', f"export AMZ_LINT={AMZ_DATA_DIR}/linter/",
                                 f"{AMZ_DATA_DIR}/amz_source.sh")
        p1 = sp.run('sudo apt update && sudo apt install --yes python-pip python3-pip yapf pylint clang-format',
                    shell=True,
                    stdout=sp.PIPE,
                    stderr=sp.PIPE)
        # Still necessary for py2 deps
        p2 = sp.run('pip install requests pyyaml', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
        p3 = sp.run(f'{AMZ_DATA_DIR}/linter/bin/init_linter_git_hooks', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
        l.end(p1.returncode + p2.returncode + p3.returncode)
        print(f"\t\t Please run {meta.highlight_cmd('source ~/.bashrc')} now")
    except:
        l.end(1)

    # Clone submodules
    l.chain("Cloning submodules", "\t", 0)
    try:
        os.chdir(amz_root_dir)
        sp.run('git submodule update --init --recursive', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
    except:
        l.end(1)

    # TODO: Clone bigfiles

    l.chain("Do you want to proceed with remaining setup? (Dependencies..)", "\t", 0)
    l.end(-3)
    INSTALL_DEPS = input('\t\t (y/n): ').lower()
    if INSTALL_DEPS == 'n':
        print("Exiting...")
        l.end(1)
        return
    elif INSTALL_DEPS not in ['y', 'n']:
        print("Didn't understand! - Exiting...")
        l.end(1)
        return

    # Repo Deps Config
    amz_repo_config = yaml.load(open(f'{amz_root_dir}/amz.yaml'), Loader=yaml.FullLoader)

    # TODO: Install ROS

    # Install apt deps
    try:
        APT_DEPS_DIR = f'{amz_root_dir}/{amz_repo_config["apt-deps"]}'
        install.apt_deps(APT_DEPS_DIR, verbose)
    except KeyError:
        l.chain("No apt dependencies file found", "\t")
        l.end(1)

    # Install python2 deps
    try:
        PY2_DEPS_DIR = f'{amz_root_dir}/{amz_repo_config["py2-deps"]}'
        install.py2_deps(PY2_DEPS_DIR, verbose)
    except KeyError:
        l.chain("No Py2 dependencies file found", "\t")
        l.end(1)

    # Install python3 deps
    try:
        PY3_DEPS_DIR = f'{amz_root_dir}/{amz_repo_config["py3-deps"]}'
        install.py3_deps(PY3_DEPS_DIR, verbose)
    except KeyError:
        l.chain("No Py3 dependencies file found", "\t")
        l.end(1)

    return


# Handles fetching of meta data
import os
from functools import reduce
from operator import getitem

import yaml


def update_line_in_file(replace_containing, replace_with, filename):
    f = open(filename, 'r')
    lines = f.readlines()
    lines_upd = [f"{replace_with}\n" if replace_containing in l else l for l in lines]
    f.close()
    with open(filename, 'w') as f:
        f.write(''.join(lines_upd))
        f.close()


def get_version():
    PKG_DIR = os.path.dirname(__file__)
    v = open(f'{PKG_DIR}/data/VERSION').readlines()[0]
    return v.ljust(10, ' ')


def get_amz_config(key_chain):

    HOME_DIR = os.environ["HOME"]
    AMZ_CONFIG_FILE = HOME_DIR + '/.amz/data/amz-config.yaml'
    try:
        config = yaml.load(open(AMZ_CONFIG_FILE, 'r'), Loader=yaml.FullLoader)
        return reduce(getitem, key_chain, config)
    except:
        return


def get_yaml_config(filepath, key_chain):

    try:
        config = yaml.load(open(filepath, 'r'), Loader=yaml.FullLoader)
        return reduce(getitem, key_chain, config)
    except:
        return


def highlight_cmd(cmd):
    return f'\033[90m\033[107m {cmd} \033[0m'



from setuptools import setup, find_packages
from datetime import date


def read_version():
    with open("amz/data/VERSION", "r") as ver:
        VER = ver.readlines()[0]
        return VER


setup(name="amz-tool",
      version=read_version(),
      author="AMZ Dev Admin",
      author_email="dev@amzracing.ch",
      description="AMZ Codebase Management Tool",
      url="https://bitbucket.org/amzracing/amz-tool",
      packages=find_packages(),
      classifiers=[
          "Programming Language :: Python :: 3", "Development Status :: 4 - Beta", "Operating System :: POSIX :: Linux",
          "License :: Other/Proprietary License"
      ],
      python_requires='>=3.6',
      include_package_data=True,
      install_requires=['pyyaml>=5.3.0', 'requests'],
      scripts=['bin/amz'])